/* Prototypes and external variable declarations.
 *
 * Services is copyright (c) 1996-1998 Andy Church.
 *     E-mail: <achurch@dragonfire.net>
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#ifndef EXTERN_H
#define EXTERN_H


#define E extern


/**** actions.c ****/

E void kill_user(const char *source, const char *user, const char *reason);
E void bad_password(User *u);


/**** channels.c ****/

E void get_channel_stats(long *nrec, long *memuse);
E Channel *findchan(const char *chan);
E Channel *firstchan(void);
E Channel *nextchan(void);

E void chan_adduser(User *user, const char *chan);
E void chan_deluser(User *user, Channel *c);

E void do_cmode(const char *source, int ac, char **av);
E void do_topic(const char *source, int ac, char **av);

E int only_one_user(const char *chan);


/**** chanserv.c ****/

E const char s_ChanServ[];

E void listchans(int count_only, const char *chan);
E void get_chanserv_stats(long *nrec, long *memuse);

E void chanserv(const char *source, char *buf);
E void load_cs_dbase(void);
E void save_cs_dbase(void);
E void check_modes(const char *chan);
E int check_valid_op(User *user, const char *chan, int newchan);
E int check_should_op(User *user, const char *chan);
E int check_should_voice(User *user, const char *chan);
E int check_kick(User *user, const char *chan);
E void record_topic(const char *chan);
E void restore_topic(const char *chan);
E int check_topiclock(const char *chan);
E void expire_chans(void);
E void cs_remove_nick(const char *nick);

E ChannelInfo *cs_findchan(const char *chan);


/**** datafiles.c ****/

E void check_file_version(FILE *f, const char *filename);
E int get_file_version(FILE *f, const char *filename);
E int write_file_version(FILE *f, const char *filename);
E FILE *open_db(const char *service, const char *filename, const char *mode);
E void close_db(FILE *dbfile, const char *filename);


/**** helpserv.c ****/

E const char s_HelpServ[];

E void helpserv(const char *whoami, const char *source, char *buf);


/**** list.c ****/

E void do_listnicks(int ac, char **av);
E void do_listchans(int ac, char **av);


/**** main.c ****/

E char *remote_server;
E int remote_port;
E char *server_name;
E char *server_desc;
E char *services_user;
E char *services_host;
E char *services_dir;
E char *log_filename;
E char *time_zone;
E int update_timeout;
E int debug;
E int readonly;
E int nofork;

E int quitting;
E int delayed_quit;
E char *quitmsg;
E char inbuf[BUFSIZE];
E int servsock;
E int save_data;
E int got_alarm;
E time_t start_time;

E void open_log(void);
E void close_log(void);
E void log(const char *fmt, ...);
E void log_perror(const char *fmt, ...);
E void fatal(const char *fmt, ...);
E void fatal_perror(const char *fmt, ...);
E void introduce_user(const char *user);


/**** memory.c ****/

E void *smalloc(long size);
E void *scalloc(long elsize, long els);
E void *srealloc(void *oldptr, long newsize);
E char *sstrdup(const char *s);


/**** memoserv.c ****/

E const char s_MemoServ[];

E void get_memoserv_stats(long *nrec, long *memuse);

E void memoserv(const char *source, char *buf);
E void load_ms_dbase(void);
E void save_ms_dbase(void);
E void check_memos(const char *nick);
E void clear_memos(const char *nick);


/**** misc.c ****/

E char *sgets2(char *buf, long size, int sock);
E char *strscpy(char *d, const char *s, size_t len);
#if !HAVE_SNPRINTF
# if BAD_SNPRINTF
#  define vsnprintf my_vsnprintf
#  define snprintf my_snprintf
# endif
E int vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
E int snprintf(char *buf, size_t size, const char *fmt, ...);
#endif
#if !HAVE_STRICMP && !HAVE_STRCASECMP
E int stricmp(const char *s1, const char *s2);
E int strnicmp(const char *s1, const char *s2, size_t len);
#endif
#if !HAVE_STRDUP
E char *strdup(const char *s);
#endif
#if !HAVE_STRSPN
E size_t strspn(const char *s, const char *accept);
#endif
E char *stristr(char *s1, char *s2);
#if !HAVE_STRERROR
E char *strerror(int errnum);
#endif
#if !HAVE_STRSIGNAL
char *strsignal(int signum);
#endif
E char *merge_args(int argc, char **argv);
E int match_wild(const char *pattern, const char *str);
E int match_wild_nocase(const char *pattern, const char *str);
E char *month_name(int month);
E char *strupper(char *s);
E char *strlower(char *s);
E char *read_string(FILE *f, const char *filename);
E void write_string(const char *s, FILE *f, const char *filename);


/**** nickserv.c ****/

E const char s_NickServ[];

E void listnicks(int count_only, const char *nick);
E void get_nickserv_stats(long *nrec, long *memuse);

E void nickserv(const char *source, char *buf);
E void load_ns_dbase(void);
E void save_ns_dbase(void);
E int validate_user(User *u);
E void cancel_user(User *u);
E void expire_nicks(void);

E NickInfo *findnick(const char *nick);


/**** operserv.c ****/

E const char s_OperServ[];
E const char s_GlobalNoticer[];

E void operserv(const char *source, char *buf);
E void load_os_dbase(void);
E void save_os_dbase(void);
E int is_services_root(const char *nick);
E int is_services_admin(const char *nick);
E void os_remove_nick(const char *nick);
E void load_akill(void);
E void save_akill(void);
E int check_akill(const char *nick, const char *username, const char *host);
E void expire_akills(void);
E void check_clones(User *user);


/**** process.c ****/

E int allow_ignore;
E IgnoreData *ignore[];

E void m_version(char *source, int ac, char **av);

E void add_ignore(const char *nick, time_t delta);
E IgnoreData *get_ignore(const char *nick);
E int split_buf(char *buf, char ***argv, int colon_special);
E void process(void);


/**** send.c ****/

E void send_cmd(const char *source, const char *fmt, ...);
E void vsend_cmd(const char *source, const char *fmt, va_list args);
E void wallops(const char *source, const char *fmt, ...);
E void notice(const char *source, const char *dest, const char *fmt, ...);
E void notice_list(const char *source, const char *dest, const char **text);
E void privmsg(const char *source, const char *dest, const char *fmt, ...);


/**** sockutil.c ****/

E int sgetc(int s);
E char *sgets(char *buf, int len, int s);
E int sread(int s, char *buf, int len);
E int sputs(char *str, int s);
E int sockprintf(int s, char *fmt,...);
E int conn(char *host, int port);
E void disconn(int s);


/**** users.c ****/

E int usercnt, opcnt, maxusercnt;

E void get_user_stats(long *nusers, long *memuse);
E User *finduser(const char *nick);
E User *firstuser(void);
E User *nextuser(void);

E void do_nick(const char *source, int ac, char **av);
E void do_join(const char *source, int ac, char **av);
E void do_part(const char *source, int ac, char **av);
E void do_kick(const char *source, int ac, char **av);
E void do_umode(const char *source, int ac, char **av);
E void do_quit(const char *source, int ac, char **av);
E void do_kill(const char *source, int ac, char **av);

E int is_oper(const char *nick);
E int is_on_chan(const char *nick, const char *chan);
E int is_chanop(const char *nick, const char *chan);
E int is_voiced(const char *nick, const char *chan);

E int match_usermask(const char *mask, User *user);
E void split_usermask(const char *mask, char **nick, char **user, char **host);
E char *create_mask(User *u);


#endif	/* EXTERN_H */
